/*
 * Copyright 2004 Device Drivers Limited.
 * Author: kawamoto@devdrv.co.jp
 *
 * arch/mips/au1000/ekit1100/rtc_4543.c
 *     low-level RTC hookups for EPSON RTC4543 chip.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/types.h>
#include <linux/time.h>

#include <asm/time.h>
#include <asm/au1000.h>
#include <asm/ekit1100.h>

#define EPOCH 2000

#undef BCD2BIN
#define BCD2BIN(val) (((val) & 15) + ((val) >> 4) * 10)
 
#undef BIN2BCD
#define BIN2BCD(val) ((((val) / 10) << 4) + (val) % 10)

static BCSR * const bcsr = (BCSR *)0xB8000000;

#define RTC_CE_SET bcsr->gpio3 |=  0x00000001
#define RTC_CE_CLR bcsr->gpio3 &= ~0x00000001
#define RTC_WR_SET bcsr->gpio3 |=  0x00000002
#define RTC_WR_CLR bcsr->gpio3 &= ~0x00000002
#define RTC_CK_SET bcsr->gpio3 |=  0x00000004
#define RTC_CK_CLR bcsr->gpio3 &= ~0x00000004
#define RTC_DT_SET bcsr->gpio3 |=  0x00000008
#define RTC_DT_CLR bcsr->gpio3 &= ~0x00000008
#define RTC_DT_OUT bcsr->gpio3 |=  0x00080000
#define RTC_DT_IN  bcsr->gpio3 &= ~0x00080000
#define RTC_DT_RD  (bcsr->gpio3 &  0x00000008)

static unsigned long rtc_4543_get_time(void)
{
	int i;
	unsigned long long temp;
	unsigned int year, month, day, hour, minute, second;
	
	RTC_WR_CLR;
	au_sync_udelay(2);
	
	RTC_DT_IN;
	au_sync_udelay(2);
	RTC_CE_SET;
	au_sync_udelay(2);
	
	temp = 0;
	for(i = 0; i < 52; i++)
	{
		temp >>= 1;
		RTC_CK_SET;
		au_sync_udelay(2);
		RTC_CK_CLR;
		au_sync_udelay(2);
		if(RTC_DT_RD) temp |= 0x8000000000000;
	}
	
	RTC_CE_CLR;
	au_sync_udelay(2);
	RTC_DT_OUT;
	au_sync_udelay(4);
	
	second = BCD2BIN(temp & 0x7F);
	minute = BCD2BIN((temp >> 8) & 0x7F);
	hour = BCD2BIN((temp >> 16) & 0x3F);
	day = BCD2BIN((temp >> 28) & 0x3F);
	month = BCD2BIN((temp >> 36) & 0x1F);
	year = BCD2BIN(((temp >> 44) & 0xFF)) + EPOCH;
	
	return mktime(year, month, day, hour, minute, second); 
}

static int rtc_4543_set_time(unsigned long t)
{
	int i;
	unsigned long long temp;
	struct rtc_time tm;
	
	to_tm(t, &tm);
	
	RTC_WR_SET;
	au_sync_udelay(2);
	
	RTC_DT_OUT;
	au_sync_udelay(2);
	RTC_CE_SET;
	au_sync_udelay(2);
	
	temp = (BIN2BCD(tm.tm_year - EPOCH) & 0xFF);
	temp <<= 8;
	temp |= (BIN2BCD(tm.tm_mon + 1) & 0x1F);
	temp <<= 8;
	temp |= (BIN2BCD(tm.tm_mday) & 0x3F);
	temp <<= 4;
	temp |= (BIN2BCD(tm.tm_wday) & 0x07);
	temp <<= 8;
	temp |= (BIN2BCD(tm.tm_hour) & 0x3F);
	temp <<= 8;
	temp |= (BIN2BCD(tm.tm_min) & 0x7F);
	temp <<= 8;
	temp |= (BIN2BCD(tm.tm_sec) & 0x7F);
	
	for(i = 0; i < 52; i++)
	{
		if(temp & 1)
			RTC_DT_SET;
		else
			RTC_DT_CLR;
		au_sync_udelay(2);
		RTC_CK_SET;
		au_sync_udelay(2);
		RTC_CK_CLR;
		au_sync_udelay(2);
		temp >>= 1;
	}
	
	RTC_CE_CLR;
	au_sync_udelay(4);
	
	return 0;
}

void rtc_4543_init(void)
{
	bcsr->gpio3 = 0x000F0000;
	
	rtc_get_time = rtc_4543_get_time;
	rtc_set_time = rtc_4543_set_time;
}
